#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "ps2.h"

extern unsigned char buf[];
unsigned char p_buf=11;			//pointer for buf[64]

unsigned char mode=RECEIVE;		//modes for keyboard
unsigned char sent_flag_key;	//1(0): key codes are (not) sent to keyboard

unsigned char keydata1;			//acknowledgment 0xFA
unsigned char keydata2;			//BAT completion 0xAA

unsigned char r_keybuf[BUFLEN];	//a ring buffer for typed keys
unsigned char p_keyB=0;			//bottom pointer for the ring buffer
unsigned char p_keyT=0;			//top pointer for the ring buffer
unsigned char keybuf[40];		//buffer for the data extracted from ring buffer

//#define DEBUG_MODE
void send_data(unsigned char data){

	unsigned char a, parity;
	int i;
	
	DATA_OUT=1;		//start bit 0V
	parity=0;

	for(i=0;i<8;i++){
		a=(data & 0x01);
		if(a==1) parity+=1;
		DATA_OUT=!a;
		data=data>>1;
		while(CLOCK_IN==0);	
		while(CLOCK_IN==1);	
	}

	if(parity & 0x01) parity=0;
	else parity=1;
	
	DATA_OUT=!parity;	//parity bit
	while(CLOCK_IN==0);	

	while(CLOCK_IN==1);
	DATA_OUT=0;	//stop bit
	while(CLOCK_IN==0);	

	while(CLOCK_IN==1);//ack bit
	while(CLOCK_IN==0);	
}

unsigned char receive_data(void){

	unsigned char a, data;
	int i;
	
	while(CLOCK_IN==1);	//start bit
	while(CLOCK_IN==0);	

	for(i=0;i<8;i++){
		data=data>>1;
		while(CLOCK_IN==1);	
		a=DATA_IN;a=a<<7;
		data+=a;
		while(CLOCK_IN==0);	
	}

	while(CLOCK_IN==1);	//parity bit
	while(CLOCK_IN==0);	

	while(CLOCK_IN==1);	//ack bit
	while(CLOCK_IN==0);	
	return(data);
}

void reset_keyboard(void){

	mode=SEND;
	keydata1=0xFF;

	DATA_OUT=1;		//start bit 0V
	while(!sent_flag_key);sent_flag_key=0;
	mode=RECEIVE2;
	while(keydata2!=0xAA);
  	  	UART2PutHex(keydata1);UART2PutChar(' ');
  	  	UART2PutHex(keydata2);UART2PrintString( " Response for reset (0xFF) \r\n" );
}

void set_leds(unsigned char leds){
	keydata1=0xED;

	mode=SEND;
	DATA_OUT=1;		//start bit 0V
	while(!sent_flag_key);sent_flag_key=0;
	mode=RECEIVE1;
	while(keydata1!=0xFA);

	keydata1=0;
	if (leds & 0x01) keydata1 |= 0x02;
	if (leds & 0x02) keydata1 |= 0x04;
	if (leds & 0x04) keydata1 |= 0x01;


	mode=SEND;
	DATA_OUT=1;		//start bit 0V
	while(!sent_flag_key);sent_flag_key=0;
	mode=RECEIVE1;
	while(keydata1!=0xFA);

	mode=RECEIVE;
}



void __attribute__((interrupt,no_auto_psv)) _INT1Interrupt(void)
{
	unsigned char rd1,rd2,rd3;

	rd1=0;rd2=0;rd3=0;

	switch(mode){
		case SEND:	//for sending LEDs data, reset data etc. to keyboard
			send_data(keydata1);
			sent_flag_key=1;
			break;

		case RECEIVE:	//for receiving typed keycodes
			rd1=receive_data();
			if(rd1==0xf0) {
				r_keybuf[p_keyT++]=0xf0;if(p_keyT==BUFLEN) p_keyT=0;
				r_keybuf[p_keyT++]=receive_data();if(p_keyT==BUFLEN) p_keyT=0;
			}
			else if (rd1==0xe0){
				rd2=receive_data();
				if(rd2==0x12)break;//ignore a sequence of (0xe0 0x12)
				else if(rd2==0xf0){	//0xe0 0xf0
					rd3=receive_data();	
					if(rd3==0x12) break;//ignore a sequence of (0xe0 0xf0 0x12)
					else {
						r_keybuf[p_keyT++]=0xe0;if(p_keyT==BUFLEN) p_keyT=0;
						r_keybuf[p_keyT++]=0xf0;if(p_keyT==BUFLEN) p_keyT=0;
						r_keybuf[p_keyT++]=rd3;if(p_keyT==BUFLEN) p_keyT=0;
					}
				}
				else {
					r_keybuf[p_keyT++]=0xe0;if(p_keyT==BUFLEN) p_keyT=0;
					r_keybuf[p_keyT++]=rd2;if(p_keyT==BUFLEN) p_keyT=0;
				}
			}
			else{
				r_keybuf[p_keyT++]=rd1;if(p_keyT==BUFLEN) p_keyT=0;
			}

			break;

		case RECEIVE1:	//for receiving acknowledgement (0xFA)
			keydata1=receive_data();
			break;

		case RECEIVE2:	//for receiving acknowledgement+BAT completion (0xFA 0xAA)
			keydata1=receive_data();
			keydata2=receive_data();
			break;

		default:
			break;
	}
   Int1_Clear_Intr_Status_Bit;
}

unsigned char extract_keys(void){

	unsigned char sp_keyB,sp_keyT,key_count;
	int i,j;

	sp_keyB=p_keyB;
	sp_keyT=p_keyT;
	p_keyB=sp_keyT;

	if(sp_keyT==sp_keyB) return 0;	//there are no data to be sent
	else if(sp_keyT>sp_keyB){
		key_count=sp_keyT-sp_keyB;
		for(i=0;i<key_count;i++) keybuf[i]=r_keybuf[sp_keyB+i];
		}
	else{
		j=BUFLEN-sp_keyB;
		key_count=j+p_keyT;
		for(i=0;i< j;i++)keybuf[i]=r_keybuf[sp_keyB+i];
		for(i=0;i<sp_keyT;i++) keybuf[i+j]=r_keybuf[i];     	
	}
	return key_count;
}

unsigned char transkey(void){
	/*
		set characters (USB HID codes: see an attached file "translate.pdf")
		in buffers (from buf[9] to buf[16])
					
		set following modiferkey in buf[9]
		ModiferKeyTable
			0x01 ; L CTRL
			0x02 ; L SHFT
			0x04 ; L ALT
			0x08 ; L GUI
			0x10 ; R CTRL
			0x20 ; R SHFT
			0x40 ; R ALT
			0x80 ; R GUI
	*/

	int i,j,k,key_count;
	unsigned char write_flag;			//1(0): should (not) be sent to PC
	static unsigned char save_buf[30];

	write_flag=0;	//0x00 or repeated key must be ignored

	if(( key_count=extract_keys() ) ==0) return 0;

	#ifdef DEBUG_MODE
	for(i=0;i<key_count;i++){
				UART2PutHex(keybuf[i]);UART2PrintString( " " );
	}
	UART2PrintString( "\r\n" );
	#endif

	for(i=0;i<key_count;i++){
		if(keybuf[i]==0xe0){	//key codes begining with 0xe0
			if(keybuf[i+1]==0xf0){
//				if(keybuf[i+2]==0x12) {i++;i++;continue;}	//ignore a sequence of (0xe0 0xf0 0x12)
//				else if((save_buf[9] & 0x04) && keybuf[i+2]==0x1f) {save_buf[9]&=0xfb;i++;i++;}
				if((save_buf[9] & 0x04) && keybuf[i+2]==0x1f) {save_buf[9]&=0xfb;i++;i++;}
				else if((save_buf[9] & 0x10) && keybuf[i+2]==0x14) {save_buf[9]&=0xef;i++;i++;}
				else if((save_buf[9] & 0x40) && keybuf[i+2]==0x11) {save_buf[9]&=0xbf;i++;i++;}
				else if((save_buf[9] & 0x80) && keybuf[i+2]==0x27) {save_buf[9]&=0x7f;i++;i++;}
				else{
					for(j=11;j<p_buf;j++) if(save_buf[j]==trans_ex(keybuf[i+2])) {i++;i++;p_buf--;break;}
					for(k=j;k<(p_buf);k++) save_buf[k]=save_buf[k+1];
					for(j=p_buf;j<30;j++) save_buf[j]=0;
				}
			}
			else if(keybuf[i+1]==0x1f) {	i++;//L ALT
				if(save_buf[9] & 0x04) continue;
				else save_buf[9]|=0x04;
			}
			else if(keybuf[i+1]==0x14) {	i++;//R CTRL
				if(save_buf[9] & 0x10) continue;
				else save_buf[9]|=0x10;
			}
			else if(keybuf[i+1]==0x11) {	i++;//R ALT
				if(save_buf[9] & 0x40) continue;
				else save_buf[9]|=0x40;
			}
			else if(keybuf[i+1]==0x27) {	i++;//R GUI
				if(save_buf[9] & 0x80) continue;
				else save_buf[9]|=0x80;
			}
//			else if(keybuf[i+1]==0x12) {i++;continue;}	//ignore a sequence of (0xe0 0x12)
			else if(save_buf[p_buf-1]==trans_ex(keybuf[i+1])) {i++;continue;}
			else {save_buf[p_buf++]=trans_ex(keybuf[i+1]);i++;}
		}
		else if(keybuf[i]==0xe1){	//Pause/Break not supported
			if(keybuf[i+1]==0x14) {	save_buf[p_buf++]=0x48; i+=2;}
			else {save_buf[p_buf-=1]=0x00;i+=4;}
		}
		else if(keybuf[i]!=0xf0){	//1Byte key code (less than 0x83)
			if(keybuf[i]==0) continue;	//ignore 0x00
			if(save_buf[p_buf-1]==trans(keybuf[i]))	continue;
			else if(keybuf[i]==0x12){	//L SHIFT
				if(save_buf[9] & 0x02) continue;
				else save_buf[9]|=0x02;
			}
			else if(keybuf[i]==0x59){	//R SHIFT
				if(save_buf[9] & 0x20) continue;
				else save_buf[9]|=0x20;
			}
			else if(keybuf[i]==0x14){	//L CTRL
				if(save_buf[9] & 0x01) continue;
				else save_buf[9]|=0x01;
			}
			else if(keybuf[i]==0x11){	//L GUI
				if(save_buf[9] & 0x08) continue;
				else save_buf[9]|=0x08;
			}
			else save_buf[p_buf++]=trans(keybuf[i]);
			for(j=p_buf;j<30;j++) save_buf[j]=0;
		}
		else{	//key codes begining with 0xf0
			if((save_buf[9] & 0x02) && keybuf[i+1]==0x12) {save_buf[9]&=0xfd;i++;}
 			else if((save_buf[9] & 0x20) && keybuf[i+1]==0x59) {save_buf[9]&=0xdf;i++;}
			else if((save_buf[9] & 0x01) && keybuf[i+1]==0x14) {save_buf[9]&=0xfe;i++;}
			else if((save_buf[9] & 0x08) && keybuf[i+1]==0x11) {save_buf[9]&=0xf7;i++;}
			else{
				for(j=11;j<p_buf;j++) if(save_buf[j]==trans(keybuf[i+1])) {i++;p_buf--;break;}
				for(k=j;k<(p_buf);k++) save_buf[k]=save_buf[k+1];
				for(j=p_buf;j<30;j++) save_buf[j]=0;


//				for(j=11;j<p_buf;j++) if(save_buf[j]==trans(keybuf[i+1])) {i++;break;}
//				if(j!=p_buf){p_buf--;
//					for(k=j;k<(p_buf);k++) save_buf[k]=save_buf[k+1];
//					for(j=p_buf;j<30;j++) save_buf[j]=0;
//				}
//				else {i++;continue;}
			}
		}
		write_flag=1;
				
	}
//sent_flag=1;
	if(write_flag==1){
		for(j=0;j<8;j++) buf[j+9]=save_buf[j+9];
		return 1;
	}
	else return 0;
}

unsigned char trans(unsigned char data){
	switch (data){
		case 0x1c: return(0x04);	//aA
		case 0x32: return(0x05);	//bB
		case 0x21: return(0x06);	//cC
		case 0x23: return(0x07);	//dD
		case 0x24: return(0x08);	//eE
		case 0x2b: return(0x09);	//fF
		case 0x34: return(0x0a);	//gG
		case 0x33: return(0x0b);	//hH
		case 0x43: return(0x0c);	//iI
		case 0x3b: return(0x0d);	//jJ
		case 0x42: return(0x0e);	//kK
		case 0x4b: return(0x0f);	//lL
		case 0x3a: return(0x10);	//mM
		case 0x31: return(0x11);	//nN
		case 0x44: return(0x12);	//oO
		case 0x4d: return(0x13);	//pP
		case 0x15: return(0x14);	//qQ
		case 0x2d: return(0x15);	//rR
		case 0x1b: return(0x16);	//sS
		case 0x2c: return(0x17);	//tT
		case 0x3c: return(0x18);	//uU
		case 0x2a: return(0x19);	//vV
		case 0x1d: return(0x1a);	//wW
		case 0x22: return(0x1b);	//xX
		case 0x35: return(0x1c);	//yY
		case 0x1a: return(0x1d);	//zZ

		case 0x16: return(0x1e);	//1!
		case 0x1e: return(0x1f);	//2"
		case 0x26: return(0x20);	//3#
		case 0x25: return(0x21);	//4$
		case 0x2e: return(0x22);	//5%
		case 0x36: return(0x23);	//6&
		case 0x3d: return(0x24);	//7'
		case 0x3e: return(0x25);	//8(
		case 0x46: return(0x26);	//9)
		case 0x45: return(0x27);	//0~??????

		case 0x5a: return(0x28);	//Enter
		case 0x76: return(0x29);	//Esc
		case 0x66: return(0x2a);	//Backspace
		case 0x0d: return(0x2b);	//Tab
		case 0x29: return(0x2c);	//Space
		case 0x4e: return(0x2d);	//-=
		case 0x55: return(0x2e);	//^~
		case 0x54: return(0x2f);	//@`
		case 0x5b: return(0x30);	//[{
		case 0x5d: return(0x31);	//]}

		case 0x4c: return(0x33);	//;+
		case 0x52: return(0x34);	//:*
		case 0x0e: return(0x35);	//1Byte or 2Bytes
		case 0x41: return(0x36);	//,<
		case 0x49: return(0x37);	//.>
		case 0x4a: return(0x38);	///?
		case 0x58: return(0x39);	//Caps Lock

		case 0x05: return(0x3a);	//F1	
		case 0x06: return(0x3b);	//F2	
		case 0x04: return(0x3c);	//F3
		case 0x0c: return(0x3d);	//F4
		case 0x03: return(0x3e);	//F5
		case 0x0b: return(0x3f);	//F6
		case 0x83: return(0x40);	//F7?????
		case 0x0a: return(0x41);	//F8
		case 0x01: return(0x42);	//F9
		case 0x09: return(0x43);	//F10
		case 0x78: return(0x44);	//F11
		case 0x07: return(0x45);	//F12

		case 0x7e: return(0x47);	//Scroll Lock

		case 0x77: return(0x53);	//Num Lock
//		case 0x52: return(0x54);	//:*
		case 0x7c: return(0x55);	//1Byte or 2Bytes
		case 0x7b: return(0x56);	//,<
		case 0x79: return(0x57);	//.>
//		case 0x4a: return(0x58);	///?
		case 0x69: return(0x59);	//Keypad 1
		case 0x72: return(0x5a);	//Keypad 2
		case 0x7a: return(0x5b);	//Keypad 3
		case 0x6b: return(0x5c);	//Keypad 4
		case 0x73: return(0x5d);	//Keypad 5
		case 0x74: return(0x5e);	//Keypad 6
		case 0x6c: return(0x5f);	//Keypad 7
		case 0x75: return(0x60);	//Keypad 8
		case 0x7d: return(0x61);	//Keypad 9
		case 0x70: return(0x62);	//Keypad 0
		case 0x71: return(0x63);	//Keypad . Delete

		case 0x51: return(0x87);	//Ro
		case 0x13: return(0x88);	//Katakana/Hiragana
		case 0x6a: return(0x89);	//\|
		case 0x64: return(0x8a);	//Henkan
		case 0x67: return(0x8b);	//Muhenkan

		default: return (data);
	}

}

unsigned char trans_ex(unsigned char data){
	switch (data){
		case 0x7c: return(0x46);	//Print Screen

		case 0x70: return(0x49);	//Insert
		case 0x6c: return(0x4a);	//Home	
		case 0x7d: return(0x4b);	//Page Up	
		case 0x71: return(0x4c);	//Delete
		case 0x69: return(0x4d);	//End
		case 0x7a: return(0x4e);	//Page Down
		case 0x74: return(0x4f);	//Right Arrow
		case 0x6b: return(0x50);	//Left Arrow
		case 0x72: return(0x51);	//Down Arrow
		case 0x75: return(0x52);	//Up Arrow

		case 0x4a: return(0x54);	//Keypad /
		case 0x5a: return(0x58);	//Keypad Enter
		case 0x2f: return(0x65);	//App

		default: return (data);
	}
}
